/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.kyuubi.plugin.spark.authz.serde

import org.apache.spark.sql.SaveMode.{Append, Overwrite}

import org.apache.kyuubi.plugin.spark.authz.PrivilegeObjectActionType._

trait ActionTypeExtractor extends (AnyRef => PrivilegeObjectActionType) with Extractor

object ActionTypeExtractor {
  val actionTypeExtractors: Map[String, ActionTypeExtractor] = {
    loadExtractorsToMap[ActionTypeExtractor]
  }
}

/**
 * Boolean
 */
class OverwriteOrInsertActionTypeExtractor extends ActionTypeExtractor {
  override def apply(v1: AnyRef): PrivilegeObjectActionType = {
    if (v1.asInstanceOf[Boolean]) INSERT_OVERWRITE else INSERT
  }
}

/**
 * org.apache.spark.sql.SaveMode
 */
class SaveModeActionTypeExtractor extends ActionTypeExtractor {
  override def apply(v1: AnyRef): PrivilegeObjectActionType = {
    v1 match {
      case Append => INSERT
      case Overwrite => INSERT_OVERWRITE
      case _ => OTHER
    }
  }
}
